local ADDON, namespace = ...
local L = namespace.L
local tooltip
local arg = {}

GMKEYSTONE_CFG = GMKEYSTONE_CFG or {} 
 
local LibQTip = LibStub('LibQTip-1.0')
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")

local bag, slot, itemID
local frame = CreateFrame("Frame")

local short = {

	[197] = "EYE" , 	-- Eye of Azhara
	[198] = "DHT", 		-- Darkhearth Thicket
	[199] = "BH", 		-- Blackrook Hold
	[200] = "HoV", 		-- Halls of Valor
	[206] = "NL", 		-- Neltharion's Lair
	[207] = "VoW", 		-- Vault of the Wardens
	[208] = "MoS", 		-- Maw of Souls
	[209] = "TARC", 	-- The Arcway
	[210] = "COURT", 	-- Court of Stars
	[227] = "LOW", 		-- Return to Karazhan: Lower
	[233] = "CATH", 	-- Cathedral of Eternal Night
	[234] = "UPPER", 	-- Return to Karazhan: Upper
	[239] = "SoT", 		-- The Seat of the Triumvirate

	[244] = "AD", 		-- Atal'Dazar
	[245] = "FH", 		-- Freehold
	[246] = "TD", 		-- Tol Dagor
	[247] = "ML", 		-- The Motherlode
	[248] = "WM", 		-- Waycrest Manor
	[249] = "KR", 		-- King's Rest
	[250] = "TOS", 		-- Temple of Sethraliss
	[251] = "UNDR", 	-- The Underrot
	[252] = "SOTS", 	-- Shrine of the Storm
	[353] = "SIEGE", 	-- Siege of Boralus
	[369] = "YARD", 	-- Operation Mechagon - Junkyard
	[370] = "WORK", 	-- Operation Mechagon - Workshop

	[375] = "MIST", 	-- Mists of Tirna Scithe
	[376] = "NW", 		-- The Necrotic Wake
	[377] = "DOS", 		-- De Other Side
	[378] = "HOA", 		-- Halls of Atonement
	[379] = "PLAGUE", 	-- Plaguefall
	[380] = "SD", 		-- Sanguine Depths
	[381] = "SPIRE", 	-- Spires of Ascension
	[382] = "TOP", 		-- Theater of Pain
	[391] = "TSOW", 	-- Tazavesh: Streets of Wonder
	[392] = "TSG", 		-- Tazavesh: So'leah's Gambit
}

local mapIDs = {
	[375] = {"MIST", 	"Mists of Tirna Scithe"},
	[376] = {"NW", 		"The Necrotic Wake"},
	[377] = {"DOS", 	"De Other Side"},
	[378] = {"HOA",		"Halls of Atonement"},
	[379] = {"PLAGUE",	"Plaguefall"},
	[380] = {"SD",		"Sanguine Depths"},
	[381] = {"SPIRE", 	"Spires of Ascension"},
	[382] = {"TOP",		"Theater of Pain"},
	[391] = {"TSOW",	"Tazavesh: Streets of Wonder"},
	[392] = {"TSG",		"Tazavesh: So'leah's Gambit"},
}

 local bigFont = CreateFont("bigFont")
 bigFont:CopyFontObject(CombatTextFont)

local dataobj = ldb:NewDataObject(ADDON, {
    type = "data source",
    icon = "Interface\\Addons\\"..ADDON.."\\icon.tga",
    text = "-"
}) 

local function updateLDB()

	if GMKEYSTONE_CFG["DISPLAY"] == "KEY" then 
		if C_MythicPlus.GetOwnedKeystoneLevel() then 
			dataobj.text = short[C_MythicPlus.GetOwnedKeystoneChallengeMapID()] .. ":" .. C_MythicPlus.GetOwnedKeystoneLevel()
		else 
			dataobj.text = "-"
		end
	else 
		dataobj.text = C_PlayerInfo.GetPlayerMythicPlusRatingSummary("player").currentSeasonScore
	end
end

local function Button_OnClick(row,arg,button)

	SendChatMessage(string.format("%s",arg.name), IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and 'INSTANCE_CHAT' or IsInRaid() and 'RAID' or IsInGroup() and 'PARTY' or 'SAY')

end

local function OnRelease(self)
	LibQTip:Release(self.tooltip)
	self.tooltip = nil
end  
 
local function OnLeave(self)
	if self.tooltip and not self.tooltip:IsMouseOver() then
		self.tooltip:Release()
	end
end  
 
local function anchor_OnEnter(self)

	arg = {}
	if self.tooltip then
		LibQTip:Release(self.tooltip)
		self.tooltip = nil  
	end

	if UnitLevel("player") == GetMaxPlayerLevel() then
	
		local row,col
		local tooltip = LibQTip:Acquire(ADDON.."tip", 4, "LEFT")
		self.tooltip = tooltip 
		tooltip:SmartAnchorTo(self)
		tooltip:EnableMouse(true)
		tooltip.OnRelease = OnRelease
		tooltip.OnLeave = OnLeave
		tooltip:SetAutoHideDelay(.1, self)
		normalFont = tooltip:GetFont()
		
		row,col = tooltip:AddLine()
		tooltip:SetCell(row,1,ADDON,"CENTER",4)
		tooltip:SetCellTextColor(row,1,1,1,0,1)
	
		row,col = tooltip:AddLine("")
		row,col = tooltip:AddLine("")
		
		for bag = 0, NUM_BAG_SLOTS do
			for slot = 1, GetContainerNumSlots(bag) do
				itemID = GetContainerItemID(bag, slot)
				if itemID then
					local _, _, _, _, _, _, itemLink, _, _, slotItemID = GetContainerItemInfo(bag, slot)
					if itemLink and itemLink:match("|Hkeystone:") then						
						row,col = tooltip:AddLine()	
						tooltip:SetCell(row,1,itemLink,"CENTER",4)
						arg[row] = { name=itemLink}
						tooltip:SetLineScript(row, 'OnMouseUp', Button_OnClick, arg[row])
					end
				end
			end
		end 

		row,col = tooltip:AddLine()
		row,col = tooltip:AddLine()

		row,col = tooltip:AddLine()
		tooltip:SetCell(row,1,"m+ score","CENTER",4)
		tooltip:SetCellTextColor(row,1,1,1,0,1)	

		row,col = tooltip:AddLine()
		tooltip:SetFont(bigFont)
		tooltip:SetCell(row,1,C_PlayerInfo.GetPlayerMythicPlusRatingSummary("player").currentSeasonScore,"CENTER",4)
		local r, g, b = C_ChallengeMode.GetDungeonScoreRarityColor(C_PlayerInfo.GetPlayerMythicPlusRatingSummary("player").currentSeasonScore):GetRGB()
		tooltip:SetCellTextColor(row,1,r,g,b,1)
		tooltip:SetFont(normalFont)

		if C_PlayerInfo.GetPlayerMythicPlusRatingSummary("player").currentSeasonScore > 0 then 

			row,col = tooltip:AddLine()
			row,col = tooltip:AddLine()
			tooltip:SetCell(row,1,"score","LEFT",1)
			tooltip:SetCellTextColor(row,1,1,1,0,1)	
			tooltip:SetCell(row,3,"F","LEFT",1)
			tooltip:SetCellTextColor(row,3,1,1,0,1)
			tooltip:SetCell(row,4,"T","LEFT",1)
			tooltip:SetCellTextColor(row,4,1,1,0,1)	
			
			tooltip:AddSeparator(1,1,1,0,1)

			for mapID in pairs(mapIDs) do
				local affix, score = C_MythicPlus.GetSeasonBestAffixScoreInfoForMap(mapID)
				if score then 
		 
					local localpos
					local r, g, b = (C_ChallengeMode.GetSpecificDungeonOverallScoreRarityColor(score)):GetRGB()
		 
					row,col = tooltip:AddLine()
					
					tooltip:SetCell(row,1,score,"LEFT",1)
					tooltip:SetCellTextColor(row,1,r,g,b,1)
					
					tooltip:SetCell(row,2,mapIDs[mapID][2],"LEFT",1)
					tooltip:SetCellTextColor(row,2,r,g,b,1)
			  
					for i,t in ipairs(affix) do
						if t.name == "Fortified" then localpos = 3 else localpos = 4 end 
						tooltip:SetCell(row,localpos,t.level,"LEFT",1)
						tooltip:SetCellTextColor(row,localpos,r,g,b,1)
					end
				end
			end
		end
		
		-- needed until I find the event triggered by score changes :)
		updateLDB()
		
		row,col = tooltip:Show()
	end
	
end
 
function dataobj.OnEnter(self)
    anchor_OnEnter(self)
end
 
function dataobj.OnLeave(self)
    -- Null operation: Some LDB displays get cranky if this method is missing.
end

dataobj.OnClick = function(self, button)  

	if InCombatLockdown() then return end

	LibQTip:Release(self.tooltip_legenda)
	self.tooltip_legenda = nil  

	LibQTip:Release(self.tooltip)
	self.tooltip = nil

	if button == "LeftButton" then 
		PVEFrame_ToggleFrame() 
	end
	
	if button == "RightButton" then 
	
		if GMKEYSTONE_CFG["DISPLAY"] == "SCORE" then 
			if C_MythicPlus.GetOwnedKeystoneLevel() then 
				dataobj.text = short[C_MythicPlus.GetOwnedKeystoneChallengeMapID()] .. ":" .. C_MythicPlus.GetOwnedKeystoneLevel()
			else 
				dataobj.text = "-"
			end
			GMKEYSTONE_CFG["DISPLAY"] = "KEY"
			print(ADDON .. ": set to show keys")
		else 
			dataobj.text = C_PlayerInfo.GetPlayerMythicPlusRatingSummary("player").currentSeasonScore
			GMKEYSTONE_CFG["DISPLAY"] = "SCORE"
			print(ADDON .. ": set to show score")
		end
	end
end

frame:RegisterEvent("PLAYER_LOGIN")
frame:RegisterEvent('BAG_UPDATE')
frame:RegisterEvent("CHALLENGE_MODE_KEYSTONE_RECEPTABLE_OPEN")
frame:SetScript("OnEvent", function(self, event, ...)

	if event == "PLAYER_LOGIN" then 
		C_Timer.After(2, function()
			C_MythicPlus.RequestCurrentAffixes()
			C_MythicPlus.RequestMapInfo()
			-- C_MythicPlus.RequestRewards()			
		end)
		
		-- Building default values 		
		local GMKEYSTONE_DEFAULTS = { DISPLAY="KEY" }
		
		for k in pairs(GMKEYSTONE_DEFAULTS) do
			if GMKEYSTONE_CFG[k] == nil then GMKEYSTONE_CFG[k] = GMKEYSTONE_DEFAULTS[k] end
		end
	end

	if event == "BAG_UPDATE" or event == "PLAYER_LOGIN" then 
		for bag = 0, NUM_BAG_SLOTS do
			for slot = 1, GetContainerNumSlots(bag) do
				itemID = GetContainerItemID(bag, slot)
				if itemID then
					local _,itemLink,_,_,_,itemType,itemSubType = GetItemInfo(itemID)
					if (itemType == "Reagent" and itemSubType == "Keystone") then
						if C_MythicPlus.GetOwnedKeystoneChallengeMapID() then
							updateLDB()
						end
					end
				end
			end
		end 
	end
	
	if event == "CHALLENGE_MODE_KEYSTONE_RECEPTABLE_OPEN" then 
		for bag = 0, NUM_BAG_SLOTS do
			for slot = 1, GetContainerNumSlots(bag) do 
				itemID = GetContainerItemID(bag, slot)
				if itemID then
					local _,_,_,_,_,itemType,itemSubType = GetItemInfo(itemID)
					if (itemType == "Reagent" and itemSubType == "Keystone") then
						C_Timer.After(1, function()
							UseContainerItem(bag, slot)
						end)
						break
					end
				end
			end
		end
	end	
end)